# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: zsearch_definitions/protocols.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='zsearch_definitions/protocols.proto',
  package='zsearch',
  syntax='proto3',
  serialized_pb=_b('\n#zsearch_definitions/protocols.proto\x12\x07zsearch*\xc6\x02\n\x08Protocol\x12\x12\n\x0ePROTO_RESERVED\x10\x00\x12\x10\n\x0cPROTO_SYSTEM\x10\x01\x12\x0e\n\nPROTO_HTTP\x10\x02\x12\x0f\n\x0bPROTO_HTTPS\x10\x03\x12\x0e\n\nPROTO_IMAP\x10\x04\x12\x0f\n\x0bPROTO_IMAPS\x10\x05\x12\x0e\n\nPROTO_SMTP\x10\x06\x12\x0f\n\x0bPROTO_SMTPS\x10\x07\x12\x0e\n\nPROTO_POP3\x10\x08\x12\x0f\n\x0bPROTO_POP3S\x10\t\x12\x10\n\x0cPROTO_MODBUS\x10\n\x12\r\n\tPROTO_FTP\x10\x0b\x12\r\n\tPROTO_SSH\x10\x0c\x12\r\n\tPROTO_DNS\x10\r\x12\r\n\tPROTO_NTP\x10\x0e\x12\x10\n\x0cPROTO_TELNET\x10\x0f\x12\x0e\n\nPROTO_UPNP\x10\x10\x12\x0e\n\nPROTO_CWMP\x10\x11\x12\x10\n\x0cPROTO_HTTP_2\x10\x12*\xaa\x08\n\x0bSubprotocol\x12\x15\n\x11SUBPROTO_RESERVED\x10\x00\x12\x14\n\x10SUBPROTO_DELETED\x10\x01\x12\x14\n\x10SUBPROTO_GENERIC\x10\x02\x12\x13\n\x0fSUBPROTO_BANNER\x10\x03\x12\x10\n\x0cSUBPROTO_TLS\x10\x04\x12\x14\n\x10SUBPROTO_TLS_1_0\x10\x05\x12\x14\n\x10SUBPROTO_TLS_1_1\x10\x06\x12\x14\n\x10SUBPROTO_TLS_1_2\x10\x07\x12\x14\n\x10SUBPROTO_TLS_1_3\x10\x08\x12\x17\n\x13SUBPROTO_HEARTBLEED\x10\t\x12\x14\n\x10SUBPROTO_CIPHERS\x10\n\x12\x12\n\x0eSUBPROTO_SSL_2\x10\x0b\x12\x12\n\x0eSUBPROTO_SSL_3\x10\x0c\x12\x10\n\x0cSUBPROTO_GET\x10\r\x12\x15\n\x11SUBPROTO_STARTTLS\x10\x0e\x12\x13\n\x0fSUBPROTO_EXPORT\x10\x0f\x12\x17\n\x13SUBPROTO_RSA_EXPORT\x10\x10\x12\x17\n\x13SUBPROTO_DHE_EXPORT\x10\x11\x12\x10\n\x0cSUBPROTO_DHE\x10\x12\x12\x12\n\x0eSUBPROTO_ECDHE\x10\x13\x12\x10\n\x0cSUBPROTO_SNI\x10\x14\x12\x13\n\x0fSUBPROTO_NO_SNI\x10\x15\x12\x11\n\rSUBPROTO_QUIC\x10\x16\x12\x11\n\rSUBPROTO_SPDY\x10\x17\x12\x10\n\x0cSUBPROTO_RSA\x10\x18\x12\x10\n\x0cSUBPROTO_DSA\x10\x19\x12\x12\n\x0eSUBPROTO_ECDSA\x10\x1a\x12\x16\n\x12SUBPROTO_DEVICE_ID\x10\x1b\x12\x1a\n\x16SUBPROTO_OPEN_RESOLVER\x10\x1c\x12\x17\n\x13SUBPROTO_OPEN_PROXY\x10\x1d\x12\x17\n\x13SUBPROTO_OPEN_RELAY\x10\x1e\x12\x11\n\rSUBPROTO_TIME\x10\x1f\x12\x19\n\x15SUBPROTO_HACKING_TEAM\x10 \x12\x1c\n\x18SUBPROTO_EXTENDED_RANDOM\x10!\x12\x16\n\x12SUBPROTO_DISCOVERY\x10\"\x12!\n\x1cSUBPROTO_SYS_PUBLIC_LOCATION\x10\xc0\x01\x12\x14\n\x0fSUBPROTO_SYS_AS\x10\xc1\x01\x12\x16\n\x11SUBPROTO_SYS_TAGS\x10\xc2\x01\x12\x1a\n\x15SUBPROTO_SYS_METADATA\x10\xc3\x01\x12\x17\n\x12SUBPROTO_SYS_WHOIS\x10\xc4\x01\x12\x1a\n\x15SUBPROTO_SYS_USERDATA\x10\xc5\x01\x12\x1b\n\x16SUBPROTO_SYS_BLACKLIST\x10\xc6\x01\x12\x1c\n\x17SUBPROTO_SYS_ALEXA_RANK\x10\xc7\x01\x12%\n SUBPROTO_SYS_RESTRICTED_LOCATION\x10\xc8\x01\x12\x19\n\x14SUBPROTO_SYS_VERSION\x10\xc9\x01\x62\x06proto3')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

_PROTOCOL = _descriptor.EnumDescriptor(
  name='Protocol',
  full_name='zsearch.Protocol',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PROTO_RESERVED', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_SYSTEM', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_HTTP', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_HTTPS', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_IMAP', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_IMAPS', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_SMTP', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_SMTPS', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_POP3', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_POP3S', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_MODBUS', index=10, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_FTP', index=11, number=11,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_SSH', index=12, number=12,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_DNS', index=13, number=13,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_NTP', index=14, number=14,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_TELNET', index=15, number=15,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_UPNP', index=16, number=16,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_CWMP', index=17, number=17,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROTO_HTTP_2', index=18, number=18,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=49,
  serialized_end=375,
)
_sym_db.RegisterEnumDescriptor(_PROTOCOL)

Protocol = enum_type_wrapper.EnumTypeWrapper(_PROTOCOL)
_SUBPROTOCOL = _descriptor.EnumDescriptor(
  name='Subprotocol',
  full_name='zsearch.Subprotocol',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_RESERVED', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_DELETED', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_GENERIC', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_BANNER', index=3, number=3,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_TLS', index=4, number=4,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_TLS_1_0', index=5, number=5,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_TLS_1_1', index=6, number=6,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_TLS_1_2', index=7, number=7,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_TLS_1_3', index=8, number=8,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_HEARTBLEED', index=9, number=9,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_CIPHERS', index=10, number=10,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SSL_2', index=11, number=11,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SSL_3', index=12, number=12,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_GET', index=13, number=13,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_STARTTLS', index=14, number=14,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_EXPORT', index=15, number=15,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_RSA_EXPORT', index=16, number=16,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_DHE_EXPORT', index=17, number=17,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_DHE', index=18, number=18,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_ECDHE', index=19, number=19,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SNI', index=20, number=20,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_NO_SNI', index=21, number=21,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_QUIC', index=22, number=22,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SPDY', index=23, number=23,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_RSA', index=24, number=24,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_DSA', index=25, number=25,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_ECDSA', index=26, number=26,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_DEVICE_ID', index=27, number=27,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_OPEN_RESOLVER', index=28, number=28,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_OPEN_PROXY', index=29, number=29,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_OPEN_RELAY', index=30, number=30,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_TIME', index=31, number=31,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_HACKING_TEAM', index=32, number=32,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_EXTENDED_RANDOM', index=33, number=33,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_DISCOVERY', index=34, number=34,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SYS_PUBLIC_LOCATION', index=35, number=192,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SYS_AS', index=36, number=193,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SYS_TAGS', index=37, number=194,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SYS_METADATA', index=38, number=195,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SYS_WHOIS', index=39, number=196,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SYS_USERDATA', index=40, number=197,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SYS_BLACKLIST', index=41, number=198,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SYS_ALEXA_RANK', index=42, number=199,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SYS_RESTRICTED_LOCATION', index=43, number=200,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SUBPROTO_SYS_VERSION', index=44, number=201,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=378,
  serialized_end=1444,
)
_sym_db.RegisterEnumDescriptor(_SUBPROTOCOL)

Subprotocol = enum_type_wrapper.EnumTypeWrapper(_SUBPROTOCOL)
PROTO_RESERVED = 0
PROTO_SYSTEM = 1
PROTO_HTTP = 2
PROTO_HTTPS = 3
PROTO_IMAP = 4
PROTO_IMAPS = 5
PROTO_SMTP = 6
PROTO_SMTPS = 7
PROTO_POP3 = 8
PROTO_POP3S = 9
PROTO_MODBUS = 10
PROTO_FTP = 11
PROTO_SSH = 12
PROTO_DNS = 13
PROTO_NTP = 14
PROTO_TELNET = 15
PROTO_UPNP = 16
PROTO_CWMP = 17
PROTO_HTTP_2 = 18
SUBPROTO_RESERVED = 0
SUBPROTO_DELETED = 1
SUBPROTO_GENERIC = 2
SUBPROTO_BANNER = 3
SUBPROTO_TLS = 4
SUBPROTO_TLS_1_0 = 5
SUBPROTO_TLS_1_1 = 6
SUBPROTO_TLS_1_2 = 7
SUBPROTO_TLS_1_3 = 8
SUBPROTO_HEARTBLEED = 9
SUBPROTO_CIPHERS = 10
SUBPROTO_SSL_2 = 11
SUBPROTO_SSL_3 = 12
SUBPROTO_GET = 13
SUBPROTO_STARTTLS = 14
SUBPROTO_EXPORT = 15
SUBPROTO_RSA_EXPORT = 16
SUBPROTO_DHE_EXPORT = 17
SUBPROTO_DHE = 18
SUBPROTO_ECDHE = 19
SUBPROTO_SNI = 20
SUBPROTO_NO_SNI = 21
SUBPROTO_QUIC = 22
SUBPROTO_SPDY = 23
SUBPROTO_RSA = 24
SUBPROTO_DSA = 25
SUBPROTO_ECDSA = 26
SUBPROTO_DEVICE_ID = 27
SUBPROTO_OPEN_RESOLVER = 28
SUBPROTO_OPEN_PROXY = 29
SUBPROTO_OPEN_RELAY = 30
SUBPROTO_TIME = 31
SUBPROTO_HACKING_TEAM = 32
SUBPROTO_EXTENDED_RANDOM = 33
SUBPROTO_DISCOVERY = 34
SUBPROTO_SYS_PUBLIC_LOCATION = 192
SUBPROTO_SYS_AS = 193
SUBPROTO_SYS_TAGS = 194
SUBPROTO_SYS_METADATA = 195
SUBPROTO_SYS_WHOIS = 196
SUBPROTO_SYS_USERDATA = 197
SUBPROTO_SYS_BLACKLIST = 198
SUBPROTO_SYS_ALEXA_RANK = 199
SUBPROTO_SYS_RESTRICTED_LOCATION = 200
SUBPROTO_SYS_VERSION = 201


DESCRIPTOR.enum_types_by_name['Protocol'] = _PROTOCOL
DESCRIPTOR.enum_types_by_name['Subprotocol'] = _SUBPROTOCOL


import abc
from grpc.early_adopter import implementations
from grpc.framework.alpha import utilities
# @@protoc_insertion_point(module_scope)



import socket
import struct
import hashlib
import json

import dateutil.parser
import dateutil.tz
import time

import zsearch_definitions.common_pb2
import zsearch_definitions.hoststore_pb2

class Metadatum(object):

    def __init__(self, key, value):
        self._pb = zsearch_definitions.common_pb2.Metadatum()
        self._pb.key = key
        self._pb.value = value

    @property
    def key(self):
        return self._pb.key

    @property
    def value(self):
        return self._pb.value

    @classmethod
    def from_dict(cls, d):
        '''Turns a dictionary into a list of Metadatum objects'''
        g = [cls(key, value) for key, value in d.iteritems()]
        return g

    @property
    def protobuf(self):
        return self._pb


class ProtocolAtom(object):

    def __init__(self, tags=None, metadata=None, data=None):

        self._pb = zsearch_definitions.hoststore_pb2.ProtocolAtom()

        # Tags
        if tags is None:
            tags = list()
        self._pb.tags.extend(sorted(tags))

        # Globals / values
        if metadata is None:
            metadata = list()
        self._metadata = sorted(metadata, key=lambda m: m.key)

        # Data
        self._data = data or {}
        self._pb.data = json.dumps(self.data, sort_keys=True)

    @property
    def metadata(self):
        return self._metadata

    @property
    def tags(self):
        return self._pb.tags

    @property
    def data(self):
        return self._data

    @property
    def protobuf(self):
        del self._pb.metadata[:]
        self._pb.metadata.extend([x.protobuf for x in self.metadata])
        return self._pb

    def calculate_sha256fp(self, serialized=None):
        if serialized is not None:
            b = serialized
        else:
            b = self.protobuf.SerializeToString()
        m = hashlib.sha1()
        m.update(b)
        return m.digest()


class Record(object):

    def __init__(self, ip, port, protocol, subprotocol,
                 domain=None, timestamp=None, protocol_atom=None,
                 scan_id=None, sha256fp=None):
        if ip is None or port is None or protocol is None or \
                subprotocol is None or protocol_atom is None or \
                scan_id is None or timestamp is None:
            raise Exception("Missing parameter")
        self._pb = zsearch_definitions.hoststore_pb2.Record()
        self._ip = ip
        self._pb.port = port
        self._pb.protocol = protocol
        self._pb.subprotocol = subprotocol
        self._pb.scanid = scan_id
        self._protocol_atom = protocol_atom

        # Set optional members
        if domain is not None:
            self._pb.domain = domain
        self._pb.sha256fp = sha256fp or protocol_atom.calculate_sha256fp()

        # We store IPs in network order whenever they're integers
        ip_bytes = socket.inet_aton(self._ip)
        (ip_net, ) = struct.unpack("I", ip_bytes)
        self._pb.ip = ip_net

        # Convert the time to an int
        dt = dateutil.parser.parse(timestamp).astimezone(dateutil.tz.tzutc())
        self._pb.timestamp = int(time.mktime(dt.timetuple()))

        # Set the protobuf atom, etc.
        self._pb.atom.CopyFrom(protocol_atom.protobuf)

    @property
    def ip(self):
        return self._ip

    @property
    def port(self):
        return self._pb.port

    @property
    def protocol(self):
        return self._protocol

    @property
    def subprotocol(self):
        return self._subprotocol

    @property
    def atom(self):
        return self._atom

    @property
    def scan_id(self):
        return self._pb.scanid

    @property
    def sha256fp(self):
        return self._pb.sha256fp

    @property
    def timestamp(self):
        return self._pb.timestamp

    @property
    def protobuf(self):
        return self._pb

